@@ -427,6 +427,28 @@ class Action:
             payload["Type"] = 1
         return self.post("OidbSvc.0xed3_1", payload)
 
+    def groupJoinAuth(self, seq: int, group: int, userid: int, cmd=None):
+        """
+        :param Seq:  GroupAdminsysnotify消息中的Seq
+        :param group: QQ群号
+        :param userid: 申请进群的QQ号
+        :param cmd: True:同意进群,False:拒绝,None:忽略
+        :return:
+        """
+        return self.post(
+            "AnswerInviteGroup",
+            {
+                "Seq": seq,
+                "Who": userid,
+                "Flag_7": 3,  # 这2个flag不知道会不会变
+                "Flag_8": 21,
+                "GroupId": group,
+                "Action": {True: 11, False: 12, None: 14}[
+                    cmd
+                ],  # 11 agree , 14 忽略 , 12/21 disagree
+            },
+        )
+
     def uploadGroupFile(
         self,
         group: int,
